"{ Package: 'stx:goodies/xtreams/core' }"

"{ NameSpace: Xtreams }"

ReadStream subclass:#PositionReadStream
	instanceVariableNames:'buffer'
	classVariableNames:''
	poolDictionaries:'Xtreams::XtreamsPool'
	category:'Xtreams-Core'
!

PositionReadStream comment:'Wraps a non-positionable stream and provides positioning capability by buffering read elements. Buffering strategy can be configured via different Buffer classes.

Instance Variables
	buffer  <Buffer> holds read elements

'
!


!PositionReadStream methodsFor:'accessing'!

get
	| object |
	buffer hasDataToRead ifTrue: [^buffer get].
	object := source get.
	buffer put: object.
	buffer readSkip: 1.
	^object
!

read: anInteger into: aSequenceableCollection at: startIndex

	^[      buffer read: anInteger into: aSequenceableCollection at: startIndex. anInteger
	] on: Incomplete do: [:exception || bufferCount sourceCount |
		bufferCount := exception count.
		sourceCount := anInteger - bufferCount.
		[source read: sourceCount into: aSequenceableCollection at: startIndex + exception count]
			on: Incomplete do: [:incomplete | sourceCount := incomplete count].
		buffer write: sourceCount from: aSequenceableCollection at: startIndex + bufferCount.
		buffer readSkip: sourceCount.
		(sourceCount + bufferCount) < anInteger ifTrue: [
			(Incomplete on: aSequenceableCollection count: sourceCount + bufferCount at: startIndex) raise].
		anInteger ]
! !

!PositionReadStream methodsFor:'initialize-release'!

buffer
	^buffer
!

buffer: aBuffer

	buffer recycle.
	buffer := aBuffer
!

close
	super close.
	buffer recycle.
	buffer := nil
!

contentsSpecies
	^source contentsSpecies
!

on: aSource
	buffer := Buffer new: DefaultBufferSize class: aSource contentsSpecies.
	super on: aSource
! !

!PositionReadStream methodsFor:'printing'!

streamingPrintOn: aStream
	super streamingPrintOn: aStream.
	aStream
		write: ' buffered: ';
		print: buffer readSize.
	buffer readSize isZero ifTrue: [^self].
	aStream
		cr; tab;
		print: buffer contentsFuture
! !

!PositionReadStream methodsFor:'seeking'!

++ anInteger
	| skipped |
	anInteger < 0 ifTrue: [ ^self -- anInteger negated ].
	skipped := buffer readSkip: anInteger.
	skipped = anInteger ifTrue: [^anInteger].
	[super ++ (anInteger - skipped)] on: Incomplete do: [:incomplete |
		(Incomplete count: incomplete count + skipped) raise].
	^anInteger
!

-- anInteger
	| skipped |
	anInteger < 0 ifTrue: [ ^self ++ anInteger negated ].
	skipped := (buffer readSkip: anInteger negated) negated.
	skipped = anInteger ifTrue: [^anInteger].
	(Incomplete count: skipped) raise
!

length
	"advance to the end"
	[ [ self ++ SmallInteger maxVal ] repeat ] on: Incomplete do: [ :ex | ].
	^buffer activeSize
!

position
	^buffer readPosition
!

position: aPosition

	| startPosition delta |
	aPosition < 0 ifTrue: [Incomplete zero raise].
	startPosition := buffer readPosition.
	delta := aPosition - startPosition.
	^[      self ++ delta. aPosition
	] on: Incomplete do: [ :ex |
		(Incomplete count: startPosition + ex count) raise ]
! !

!PositionReadStream methodsFor:'testing'!

isPositionable
	^true
! !

!PositionReadStream methodsFor:'transforming'!

positioning
	^self
! !

!PositionReadStream class methodsFor:'documentation'!

version_SVN
    ^ '$Id$'
! !
