"{ Package: 'stx:goodies/xtreams/transforms' }"

"{ NameSpace: Xtreams }"

ReadStream subclass:#ObjectAnalyseStream
	instanceVariableNames:'marshaler nothing log depth objects int8 int16 int32 int64 uint16
		uint32 uint64 float double'
	classVariableNames:''
	poolDictionaries:''
	category:'Xtreams-Transforms'
!


!ObjectAnalyseStream class methodsFor:'instance creation'!

on: aDestination
	^self on: aDestination marshaler: ObjectMarshaler new
!

on: aDestination marshaler: aMarshaler
	^self new on: aDestination marshaler: aMarshaler
! !

!ObjectAnalyseStream methodsFor:'accessing'!

get
	| details |
	marshaler analyse: self.
	details := log conclusion.
	log := String new writing.
	self complete.
	^details
!

read: anInteger into: aSequenceableCollection at: startIndex
	| count |
	count := 0.
	[anInteger timesRepeat:
		[marshaler analyse: self.
		aSequenceableCollection at: startIndex + count put: log conclusion.
		log := String new writing.
		self complete.
		count := count + 1]]
			on: Incomplete do: [:exception |
				aSequenceableCollection at: startIndex + count put: log conclusion.
				log := String new writing.
				self complete.
				(Incomplete on: aSequenceableCollection count: count at: startIndex) raise].
	^anInteger
! !

!ObjectAnalyseStream methodsFor:'initialize-release'!

contentsSpecies
	^Array
!

on: aSource marshaler: aMarshaler
	super on: aSource.
	marshaler := aMarshaler.
	nothing := Object new.
	objects := OrderedCollection with: Transcript.
	depth := 0.

	log := String new writing.
	int8 := aSource interpreting: #signedChar.
	(self log: 'header' do: [marshaler configureAnalyse: aSource])
		ifTrue: [
			int16 := aSource interpreting: #signedShort_be.
			int32 := aSource interpreting: #signedLong_be.
			int64 := aSource interpreting: #signedLonglong_be.
			uint16 := aSource interpreting: #unsignedShort_be.
			uint32 := aSource interpreting: #unsignedLong_be.
			uint64 := aSource interpreting: #unsignedLonglong_be]
		ifFalse: [
			int16 := aSource interpreting: #signedShort_le.
			int32 := aSource interpreting: #signedLong_le.
			int64 := aSource interpreting: #signedLonglong_le.
			uint16 := aSource interpreting: #unsignedShort_le.
			uint32 := aSource interpreting: #unsignedLong_le.
			uint64 := aSource interpreting: #unsignedLonglong_le].
	float := aSource interpreting: #float.
	double := aSource interpreting: #double
! !

!ObjectAnalyseStream methodsFor:'primitives'!

double
	^double
!

float
	^float
!

int16
	^int16
!

int32
	^int32
!

int64
	^int64
!

int8
	^int8
!

uint16
	^uint16
!

uint32
	^uint32
!

uint64
	^uint64
!

uint8
	^source
! !

!ObjectAnalyseStream methodsFor:'private'!

complete
	objects := OrderedCollection with: Transcript
!

log: type do: aBlock
	| superlog sublog position incomplete |
	position := source position.
	superlog := log.
	log := String new writing.

	depth := depth + 1.
	incomplete := false.
	^[aBlock on: Incomplete do: [:ex | incomplete := true. ex pass]] ensure:
		[depth := depth - 1.
		sublog := log conclusion.
		log := superlog.

		incomplete ifFalse:
			[log tab: depth; print: position; write: '+'; print: (source position - position); tab.
			type isCharacters
				ifTrue:	[log write: type]
				ifFalse:	[log write: type value].
			log cr; write: sublog]]
!

nothing
	^nothing
!

objects
	^objects
! !

!ObjectAnalyseStream class methodsFor:'documentation'!

version_SVN
    ^ '$Id$'
! !
